#!/bin/bash

# Parse p0f fingerprint data and insert into mysql db. 

# Last modified: August 21, 2003 by Marion Bates <mbates@whoopis.com>

# PREP:
# 
# 1. mysql installed and working, FILE privileges for the mysql account
# used by this script (if that's too hard, then let the script connect as root?)
#
# 2. create database p0f;
#
#	use p0f
#
#	create table fingerprints ( 
#	id int(11) not null auto_increment primary key, 
#	windowSize varchar(4),
#	ttl int(4), 
#	fragment enum("0","1"), 
#	synSize int(4), 
#	options varchar(20), 
#	OSName text, 
#	OSDetails varchar(30) 
#	);
# 
# 3. Make sure p0f.fp is world-readable, and is in the same directory as this script
#
# 4. Run like this:
#
#	./p0f2mysql p0f.fp 
#

# Set this stuff accordingly

DBUSER="root"
DBPASS="pAsSwOrD"		
DATABASE="p0f"

# Edit accordingly
HOME=/Users/mbates/WORK/p0f_stuff/mystuff

# Make sure we were given a file
# $1 always equals name of file passed to script...bash feature

if [ -z $1 ]; then

	echo -e "\nNo filename specified. Usage: ./p0f2mysql.sh name-of-fingerprint-file"
	exit 1
fi

# Test passed? Then proceed.

# Define the fingerprint file

FPFILE=$1

# Gaaah! Stupid Mac linefeeds fubar'd this whole deal.
# So we're gonna run 'em through the linefeed fixer every time, just in case.  :)
exec `perl -pi -e 's/\r\n?/\n/g' $FPFILE`

# Get rid of comments...
grep -v "#" $FPFILE > $FPFILE.no-comments

# Get rid of the blank lines...
grep -v ^$ $FPFILE.no-comments > $FPFILE.no-blanks

# Clean up and rename
rm $FPFILE.no-comments
mv $FPFILE.no-blanks $FPFILE.ready4mysql

# Don't indent the stuff between EOFs or it fails

# Back up what's there, in case there's a major problem with the load

# Generate a unique ID number 

UNIQUEID=`mktemp -q XXXXXX`
if [ $? -ne 0 ]; then
        echo "$0: Can't create file XXXXXX, exiting..." >&2
        exit 1
fi

mkdir $HOME/p0f-db-backup.$UNIQUEID	
# This way we won't overwrite old backups

chmod 777 $HOME/p0f-db-backup.$UNIQUEID	
# So mysql can write to it

mysql -u "$DBUSER" -p"$DBPASS" "$DATABASE" <<EOF

select * from fingerprints into outfile '$HOME/p0f-db-backup.$UNIQUEID/p0f-db-backup.txt'

EOF

# Then delete old fingerprint data entries, if there are any, 
# and reset the auto_increment id so it starts from 1 again
#
# This is important because as fingerprints are added, the number will change,
# and I really hope that when they get added, they get APPENDED AT THE END 
# of the existing list -- otherwise we can't use id as a foreign key in another
# table, which would be ideal for log cross-referencing!

mysql -u "$DBUSER" -p"$DBPASS" "$DATABASE" <<EOF

truncate table fingerprints

EOF

# Load 'em

mysql -u "$DBUSER" -p"$DBPASS" "$DATABASE" <<EOF

load data local infile '$FPFILE.ready4mysql' into table fingerprints 
fields terminated by ':' 
lines starting by '' terminated by '\n' 
(windowSize, ttl, fragment, synSize, options, OSName, OSDetails)

EOF

# Figure out how many fingerprints were loaded

FPNUM=`mysql -u "$DBUSER" -p"$DBPASS" "$DATABASE" <<EOF

select count(*) from fingerprints

EOF
`	# This trailing backtic is needed here...do not delete

# Strip out the rest of mysql's answer, we just want the number
FPSLOADED=`echo $FPNUM | awk '{ print $2 }'`

echo -e "\nDone."

echo -e "\n$FPSLOADED fingerprints have been loaded into the database."

# Count lines in the original file for error-check
FPLINECOUNT=`wc -l $FPFILE.ready4mysql | awk '{ print $1 }'` 

echo -e "\nI counted $FPLINECOUNT fingerprints in the original file. These numbers should match. If they don't, then there was an error..."

# Finish cleaning up, and exit
rm $FPFILE.ready4mysql
rm $UNIQUEID
exit 0;
	
# end of script